@@ -1,4 +1,7 @@
-module.exports = function(DOMPurify, tests, xssTests) {
+module.exports = function(DOMPurify, window, tests, xssTests) {
+  var document = window.document;
+  var jQuery = window.jQuery;
+
   QUnit
     .cases(tests)
     .test( 'Sanitization test', function(params, assert) {
@@ -75,9 +78,10 @@ module.exports = function(DOMPurify, tests, xssTests) {
           ["<b>  </b>", "<b> </b>", "<b> <form><img src=\"x\"></form> </b>"] 
       );
       assert.contains( DOMPurify.sanitize( '<b>he{{evil<script>alert(1)</script><form><img src=x name=textContent></form>}}ya</b>', {SAFE_FOR_TEMPLATES: true}), 
-          ["<b>he  ya</b>", "<b>he </b>", "<b>he <form><img src=\"x\"></form> ya</b>"] // Investigate on Safari 8! 
+          ["<b>he  ya</b>", "<b>he </b>", "<b>he <form><img src=\"x\"></form> ya</b>"]
       );
       assert.equal( DOMPurify.sanitize( '<a>123<% <b>456}}</b><style>{{ alert(1) }}</style>456 %></a>', {SAFE_FOR_TEMPLATES: true}), "<a>123 <b> </b><style> </style> </a>" );
+      assert.equal( DOMPurify.sanitize( '<a href="}}javascript:alert(1)"></a>', {SAFE_FOR_TEMPLATES: true}), "<a></a>" );
   }); 
   QUnit.test( 'Config-Flag tests: SANITIZE_DOM', function(assert) {
       // SANITIZE_DOM
@@ -259,4 +263,97 @@ module.exports = function(DOMPurify, tests, xssTests) {
       assert.equal(modified, DOMPurify.sanitize(dirty));
       DOMPurify.removeHooks('afterSanitizeElements')
   } );
+  QUnit.test( 'sanitize() should allow unknown protocols when ALLOW_UNKNOWN_PROTOCOLS is true', function (assert) {
+      var dirty = '<div><a href="spotify:track:12345"><img src="cid:1234567"></a></div>';
+      assert.equal(dirty, DOMPurify.sanitize(dirty, {ALLOW_UNKNOWN_PROTOCOLS: true}));
+  } );
+
+  QUnit.test( 'sanitize() should not allow javascript when ALLOW_UNKNOWN_PROTOCOLS is true', function (assert) {
+      var dirty = '<div><a href="javascript:alert(document.title)"><img src="cid:1234567"/></a></div>';
+      var modified = '<div><a><img src="cid:1234567"></a></div>';
+      assert.equal(modified, DOMPurify.sanitize(dirty, {ALLOW_UNKNOWN_PROTOCOLS: true}));
+  } );
+
+  QUnit.test( 'Regression-Test to make sure #166 stays fixed', function (assert) {
+      var dirty = '<p onFoo="123">HELLO</p>';
+      var modified = '<p>HELLO</p>';
+      assert.equal(modified, DOMPurify.sanitize(dirty, {ALLOW_UNKNOWN_PROTOCOLS: true}));
+  } );  
+
+  // Test 1 to check if the element count in DOMPurify.removed is correct
+  QUnit.test( 'DOMPurify.removed should contain one element', function (assert) {
+      var dirty = '<svg onload=alert(1)><filter><feGaussianBlur /></filter></svg>';
+      DOMPurify.sanitize(dirty);
+      assert.equal(DOMPurify.removed.length, 1);
+  } );
+
+  // Test 2 to check if the element count in DOMPurify.removed is correct
+  QUnit.test( 'DOMPurify.removed should contain two elements', function (assert) {
+      var dirty = '1<script>alert(1)<\/script><svg onload=alert(1)><filter><feGaussianBlur /></filter></svg>';
+      DOMPurify.sanitize(dirty);
+      assert.equal(DOMPurify.removed.length, 2);
+  } );
+
+  // Test 3 to check if the element count in DOMPurify.removed is correct
+  QUnit.test( 'DOMPurify.removed should be correct', function (assert) {
+      var dirty = '<img src=x onerror="alert(1)">';
+      DOMPurify.sanitize(dirty);
+      assert.equal(DOMPurify.removed.length, 1);
+  } );
+
+  // Test 4 to check that DOMPurify.removed is correct in SAFE_FOR_TEMLATES mode 
+  QUnit.test( 'DOMPurify.removed should be correct in SAFE_FOR_TEMPLATES mode', function (assert) {
+      var dirty = '<a>123{{456}}</a>';
+      DOMPurify.sanitize(dirty, {WHOLE_DOCUMENT: true, SAFE_FOR_TEMPLATES: true});
+      assert.equal(DOMPurify.removed.length, 1);
+  } );
+
+  // Test 5 to check that DOMPurify.removed is correct in SAFE_FOR_TEMLATES mode 
+  QUnit.test( 'DOMPurify.removed should be correct in SAFE_FOR_TEMPLATES mode', function (assert) {
+      var dirty = '<a>123{{456}}<b>456{{789}}</b></a>';
+      DOMPurify.sanitize(dirty, {WHOLE_DOCUMENT: true, SAFE_FOR_TEMPLATES: true});
+      assert.equal(DOMPurify.removed.length, 2);
+  } );
+
+  // Test 6 to check that DOMPurify.removed is correct in SAFE_FOR_TEMLATES mode 
+  QUnit.test( 'DOMPurify.removed should be correct in SAFE_FOR_TEMPLATES mode', function (assert) {
+      var dirty = '<img src=1 width="{{123}}">';
+      DOMPurify.sanitize(dirty, {WHOLE_DOCUMENT: true, SAFE_FOR_TEMPLATES: true});
+      assert.equal(DOMPurify.removed.length, 1);
+  } );
+
+  // Test 7 to check that DOMPurify.removed is correct in SAFE_FOR_JQUERY mode 
+  QUnit.test( 'DOMPurify.removed should be correct in SAFE_FOR_JQUERY mode', function (assert) {
+      var dirty = '<option><iframe></select><b><script>alert(1)<\/script>';
+      DOMPurify.sanitize(dirty, {SAFE_FOR_JQUERY: true});
+      assert.equal(DOMPurify.removed.length, 2);
+  } );
+
+  // Test 8 to check that DOMPurify.removed is correct if tags are clean 
+  QUnit.test( 'DOMPurify.removed should not contain elements if tags are permitted', function (assert) {
+      var dirty = '<a>123</a>';
+      DOMPurify.sanitize(dirty);
+      assert.equal(DOMPurify.removed.length, 0);
+  } );
+
+  // Test 9 to check that DOMPurify.removed is correct if the tags and attributes are clean
+  QUnit.test( 'DOMPurify.removed should not contain elements if all tags and attrs are permitted', function (assert) {
+      var dirty = '<img src=x>';
+      DOMPurify.sanitize(dirty);
+      assert.equal(DOMPurify.removed.length, 0);
+  } );
+
+  // Test 10 to check that DOMPurify.removed does not have false positive elements in SAFE_FOR_TEMLATES mode
+  QUnit.test( 'DOMPurify.removed should not contain elements for valid data in SAFE_FOR_TEMLATES mode', function (assert) {
+      var dirty = '1';
+      DOMPurify.sanitize(dirty, {WHOLE_DOCUMENT: true, SAFE_FOR_TEMPLATES: true});
+      assert.equal(DOMPurify.removed.length, 0);
+  } );
+
+  // Test 11 to check that DOMPurify.removed does not have false positive elements in SAFE_FOR_JQUERY mode 
+  QUnit.test( 'DOMPurify.removed should not contain elements for valid data in SAFE_FOR_JQUERY mode', function (assert) {
+      var dirty = '1';
+      DOMPurify.sanitize(dirty, {WHOLE_DOCUMENT: true, SAFE_FOR_JQUERY: true});
+      assert.equal(DOMPurify.removed.length, 0);
+  } );
 }
